#ifndef S3D_ARRAY_H
#define S3D_ARRAY_H

#include <s3d/error.h>

namespace Sengir3D{

template<typename T,uint size> class Array{
	protected:  T element[size];
	public:     inline uint GetNumber() const{ return size; }
	            inline void Construct(){}
	            inline void Destruct(){}
	            inline void Set(uint nr,const T newelement){
	            	assert(nr < size);
	            	element[nr]=newelement;
	            }
	            inline Reference GetReference(const T e,Enum parameter=REQUIRED) const{
	            	for(uint c=0;c<size;c++) if(element[c]==e) return c;
	            	if(parameter==REQUIRED) throw ElementNotFound(); else return INVALID;
	            }
	            template<uint size2> inline Array<T,size> &operator=(const Array<T,size2> &newarray){
	            	if(this != &newarray){
	            		assert(newarray.GetNumber() == size);
	            		memcpy(element,newarray.element,size*sizeof(T));
	            	}
	            }
	            inline Array<T,size> &operator=(const Array<T,size> &newarray){
	            	if(this != &newarray) memcpy(element,newarray.element,size*sizeof(T));
	            	return *this;
	            }
	            template<uint size2> inline bool operator==(const Array<T,size2> &array2) const{
	            	if(array2.GetNumber() != size) return false;
	            	for(uint c=0;c<size;c++){
	            		if(element[c]!=array2[c]) return false;
	            	}
	            	return true;
	            }
	            template<uint size2> inline bool operator!=(const Array<T,size2> &array2) const{
	            	return !(operator==(array2));
	            }
	            inline void Exchange(Reference ref1,Reference ref2){
	            	assert(ref1 < size);
	            	assert(ref2 < size);
	            	byte temp[sizeof(T)];
	            	memcpy(temp,element+ref1,sizeof(T));
	            	memcpy(element+ref1,element+ref2,sizeof(T));
	            	memcpy(element+ref2,temp,sizeof(T));
	            }
	            inline Boolean Contains(const T &e) const{
	            	for(uint c=0;c<size;c++) if(element[c]==e) return YES;
	            	return NO;
	            }
	            inline Array(){}
	            inline const T *Get() const{ return element; }
	            inline const T &Get(uint nr) const{
	            	assert(nr < size);
	            	return element[nr];
	            }
	            inline const T &operator[](uint i) const{
	            	assert(i<size);
	            	return element[i];
	            }
	            inline T &operator[](uint i){
	            	assert(i<size);
	            	return element[i];
	            }
	            inline operator const T *() const{ return element; }
	            inline operator T *(){ return element; }
	            inline operator const void *() const{ return element; }
	            inline operator void *(){ return element; }
	            template<uint size2> inline Array(const Array<T,size2> &newarray){ operator=(newarray); }
	            inline Array(const Array<T,size> &newarray){ operator=(newarray); }
};

template<typename T> class Array<T,DYNAMIC>{
	protected: uint number,allocated;
	           T *element;
	public:  inline uint GetNumber() const { return number; }
	         inline void Clear(){ number=0; }
	         inline void Construct(){}
	         inline void Destruct(){ Clear(); }
	         inline void Free(){
	         	Clear();
	         	allocated=0;
	         	if(element){
	         		free(element);
	         		element=NULL;
	         	}
	         }
	private: void Reallocate(uint newnumber){
	         	element=(T *)realloc(element,sizeof(T)*newnumber);
	         	assert(element);
	         	allocated=newnumber;
	         }
	         inline void Resize(uint newnumber){
	         	if(newnumber>allocated) Reallocate(newnumber);
	         	number=newnumber;
	         }
	public:  inline void Add(const T newelement){
	         	Resize(number+1);
	         	element[number-1]=newelement;
	         }
	         template<uint size> inline void Add(const Array<T,size> &newarray){
	         	for(uint c=0;c<newarray.GetNumber();c++) Add(newarray[c]);
	         }
	         inline void Set(uint nr,const T newelement){
	         	assert(nr<number);
	         	element[nr]=newelement;
	         }
	         inline void RemoveElement(uint nr){
	         	element[nr]=element[number-1];
	         	number--;
	         }
	         inline void Remove(const T oldelement){
	         	for(int c=number-1;c>=0;c--){
	         		if(element[c]==oldelement){
	         			RemoveElement(c);
	         			return;
	         		}
	         	}
	         	throw ElementNotFound();
	         }
	         inline Reference GetReference(const T e,Enum parameter=REQUIRED) const {
	         	for(int c=number-1;c>=0;c--) if(element[c]==e) return c;
	         	if(parameter==REQUIRED) throw ElementNotFound(); else return INVALID;
	         }
	         template<uint size> inline Array<T,DYNAMIC> &operator= (const Array<T,size> &newarray){
	         	if(&newarray!=this){
	         		Resize(newarray.GetNumber());
	         		memcpy(element,newarray.element,number*sizeof(T));
	         	}
	         	return *this;
	         }
	         inline Array<T,DYNAMIC> &operator= (const Array<T,DYNAMIC> &newarray){
	         	if(this != &newarray){
	         		Resize(newarray.GetNumber());
	         		memcpy(element,newarray.element,number*sizeof(T));
	         	}
	         	return *this;
	         }
	         template<uint size> inline bool operator ==(const Array<T,size> &array2) const{
	         	if(number!=array2.number) return false;
	         	for(uint c=0;c<number;c++){
	         		if(element[c]!=array2[c]) return false;
	         	}
	         	return true;
	         }
	         template<uint size> inline bool operator !=(const Array<T,size> &array2) const{
	         	return !(operator==(array2));
	         }
	         inline void SetNumber(uint newnumber){
	         	Resize(newnumber);
	         }
	         inline void SetMinNumber(uint minnumber){
	         	if(number<minnumber){
	         		Resize(minnumber);
	         	}
	         }
	         inline void operator++ (int){
	         	Resize(number+1);
	         }
	         inline void operator+=(uint add){
	         	Resize(number+add);
	         }
	         inline void operator-- (int){
	         	assert(number>0);
	         	number--;
	         }
	         inline void operator-=(uint sub){
	         	assert(number>=sub);
	         	number-=sub;
	         }
	         void Insert(uint nr,const T newelement){
	         	if(nr>=number){
	         		SetNumber(nr+1);
	         		element[nr]=newelement;
	         	}else{
		        	SetNumber(number+1);
		        	for(int c=number-2;c>=(int)nr;c--) element[c+1]=element[c];
		        	element[nr]=newelement;
		       	}
	         }
	         void Insert(const T element1,const T element2,const T newelement){
	         	uint v1,v2;
	         	for(v1=0;v1<number;v1++){
	         		if(v1!=number-1) v2=v1+1; else v2=0;
	         		if(((element[v1]==element1)&&(element[v2]==element2))||((element[v1]==element2)&&(element[v2]==element1))){
	         			return Insert(v1+1,newelement);
	         		}
	         	}
	         	throw ElementNotFound();
	         }
	         inline void Exchange(Reference ref1,Reference ref2){
	         	assert(ref1 < number);
	         	assert(ref2 < number);
	         	byte temp[sizeof(T)];
	         	memcpy(temp,element+ref1,sizeof(T));
	         	memcpy(element+ref1,element+ref2,sizeof(T));
	         	memcpy(element+ref2,temp,sizeof(T));
	         }
	         inline Boolean Contains(const T &e) const{
	         	for(int c=number;c>=0;c--) if(element[c]==e) return YES;
	         	return NO;
	         }
	         inline Array() : number(0),allocated(0),element(NULL){}
	         inline void Initialize(){
	         	number=allocated=0;
	         	element=NULL;
	         }
	         inline const T *Get() const{ return element; }
	         inline const T &Get(uint nr) const{
	         	assert(nr<number);
	         	return element[nr];
	         }
	         inline const T &operator[](uint i) const{
	         	assert(i<number);
	         	return element[i];
	         }
	         inline T &operator[](uint i){
	         	assert(i<number);
	         	return element[i];
	         }
	         inline operator const T *() const{ return element; }
	         inline operator T *() { return element; }
	         inline operator const void *() const{ return element; }
	         inline operator void *(){ return element; }
	         template<uint size> inline Array(const Array<T,size> &newarray) : number(0),allocated(0),element(NULL){
	         	operator=(newarray);
	         }
	         inline Array(const Array<T,DYNAMIC> &newarray) : number(0),allocated(0),element(NULL){
	         	operator=(newarray);
	         }
	         inline explicit Array(uint newnumber) : number(0),allocated(0),element(NULL){
	         	SetNumber(newnumber);
	         }
	         inline Array(uint newnumber,T *newelement) : number(newnumber),allocated(newnumber){
	         	if(number>0){
		       		assert(newelement);
		       		element=malloc(sizeof(T)*number);
		       		assert(element);
		       		memcpy(element,newelement,sizeof(T)*number);
		       	}else element=NULL;
	         }
	         inline ~Array(){
	         	Free();
	         }
};



}
#endif //S3D_ARRAY_H

